package com.buy507.mall.mapper;

import java.util.List;

import org.apache.ibatis.annotations.Param;

import com.buy507.mall.model.DmsMemberBankAccount;

public interface DmsMemberBankAccountMapper {

	List<DmsMemberBankAccount> selectByMemberId(Long memberId);
	
	int insert(DmsMemberBankAccount memberBankAccount);
	
	int updateById(DmsMemberBankAccount memberBankAccount);
	
	int deleteById(Long id);
	
	DmsMemberBankAccount selectOneById(Long id);
	
	DmsMemberBankAccount selectOneByMemberIdAndDefaultStatus(@Param("memberId") Long memberId, @Param("defaultStatus") Integer defaultStatus);
	
}
